# Define function PartialEvsiBKVersusMCMC
# This function runs EVSI on the model using both B&K and OpenBUGS 2 level for one chosen design

PartialEvsiBKVersusMCMC <- function(kdraw, jdraw, nburnin, n1, n2, n3)
{# this program now does both the 2 level evsi and the Laplace approximation (1st and 2nd order)
# this enables us to take out any noise when we compare the two methods i.e. each estimate is based on the same kdraw sets of sample data
# The program also report the estimate of the inner integral using MonteCarlo, 1st rder Laplace and 2nd order Laplace for both net benefit functions
# theta0 is the prior mean
# sigma0 is the prior var-covar matrix
# lambda is threshold cost per QALY
# baseline is the expected net benefit of the baseline decision
# kdraw is the number of outer iterations
# jdraw is the number of inner iterations
# note we had ndraw in the original Laplace analysis , but ndraw is just i draw i.e. the number of datasets sampled

    NB1 <- rep(0, len = jdraw)                  # values of net benefit for treatment 1 for each inner loop in Two level
    NB2 <- rep(0, len = jdraw)                  # values of net benefit for treatment 1 for each inner loop in Two level
    TwolevelmaxNB <- rep(0, len = kdraw)        # renamed maxNB <- rep(0, len = kdraw) to TwolevelmaxNB
    firstordermaxNB <- rep(0, len = kdraw)      
    secondordermaxNB <- rep(0, len = kdraw)

#define other variables
    MonteCarloMean1 <- rep(0, len = kdraw)
    MonteCarloMean2 <- rep(0, len = kdraw)
    firstorder1 <- rep(0, len = kdraw)
    firstorder2 <- rep(0, len = kdraw)
    secondorder1 <- rep(0, len = kdraw)
    secondorder2 <- rep(0, len = kdraw)
    timeforBUGSrun <- rep(0, len = kdraw)
    timeforBKrun <- rep(0, len = kdraw)

#start outer loop

        for(i in 1:kdraw) {

        # print the iteration to see progress
        print(i)
        print(date())
        #flush.console() ensures the program writes any printed material to the screen
        #as and when asked rather than waiting to the end of the expression
        flush.console()

	  #sample from theta and generate data sample
        
	  theta <- mvrnorm(1, mu = theta0, Sigma = sigma0)
        data <- data.sample(theta, n1, n2, n3)

	  #select out the data, prior mean, and precision matrices that relate to theta 5,6,7,14,15,16

        data$dataM1<-data$newdata[,c(6,15)]
	  data$dataM2<-data$newdata[,c(5,7,14,16)]
	  data$theta0M1<-data$theta0[c(6,15)]
	  data$theta0M2<-data$theta0[c(5,7,14,16)]
	  data$sigmainvM1<-data$sigmainverse[c(6,15),c(6,15)]
	  data$sigmainvM2<-data$sigmainverse[c(5,7,14,16),c(5,7,14,16)]
	  data$sigma0invM1<-data$sigma0inverse[c(6,15),c(6,15)]
	  data$sigma0invM2<-data$sigma0inverse[c(5,7,14,16),c(5,7,14,16)]


time1 <- Sys.time() # START time to run BUGS

nposteriorsamples<-jdraw
nburnin<-nburnin

UnbalancedR2OpenBUGSfunctionResult<-UnbalancedR2OpenBUGSfunction(nposteriorsamples, nburnin, data)

# this code runs the inner loop
# calculate the inner integral using Monte Carlo with jdraw iterations
        for(j in 1:jdraw) {

		theta<-data$theta0
            theta[c(5:7,14:16)] <- UnbalancedR2OpenBUGSfunctionResult$UnbalancedBUGSresult.params$theta[j,]

            NB1[j] <- v1(theta, lambda)
            NB2[j] <- v2(theta, lambda)
            }
        MonteCarloMean1[i] <- mean(NB1)
        MonteCarloMean2[i] <- mean(NB2)  
        TwolevelmaxNB[i] <- max(MonteCarloMean1[i], MonteCarloMean2[i])

time2 <- Sys.time() # END time to BUGS and START time to run B&K
timeforBUGSrun[i] <- as.numeric((time2-time1), units="secs")

#  calculate the inner integral using B&K first order
#  calculate the inner integral using B&K second order

	nlmdetails <- nlm(negative.log.posterior, theta0, data=data, hessian = TRUE)
	thetahat <- nlmdetails$estimate
	J <- nlmdetails$hessian
	code <- nlmdetails$code # an integer indicating why the optimization process terminated
        thetaminus <- thetaplusminus(-1, thetahat, data, J)
        thetaplus <- thetaplusminus(1, thetahat, data, J)
        local <- alphaplusminus(thetaminus, thetaplus, data)
        alphaminus<- local$alphaminus
        alphaplus <- local$alphaplus
        expectation1 <- SKexpectation(v1, lambda, thetahat, thetaminus, thetaplus, alphaminus, alphaplus)
        expectation2 <- SKexpectation(v2, lambda, thetahat, thetaminus, thetaplus, alphaminus, alphaplus)
        firstorder1[i] <- expectation1$firstorder              
        firstorder2[i] <- expectation2$firstorder              
        firstordermaxNB[i] <- max(firstorder1[i], firstorder2[i])
        secondorder1[i] <- expectation1$secondorder          
        secondorder2[i] <- expectation2$secondorder           
        secondordermaxNB[i] <- max(secondorder1[i], secondorder2[i])

time3 <- Sys.time() # END time to B&K
timeforBKrun[i] <- as.numeric((time3-time2), units="secs")

    }   #   finish outer loop
    TwolevelBUGSEVSI <- mean(TwolevelmaxNB) - baseline
    FirstorderBKEVSI <- mean(firstordermaxNB) - baseline
    SecondorderBKEVSI <- mean(secondordermaxNB) - baseline
    TwolevelWinBUGSEVSIindex <- TwolevelWinBUGSEVSI/overallevpi
    FirstorderBKEVSIindex <- FirstorderBKEVSI/overallevpi
    SecondorderBKEVSIindex <- SecondorderBKEVSI/overallevpi
    BKversusTwoLevelRatio<- SecondorderBKEVSI/TwolevelWinBUGSEVSI
    
    BUGStiming <- sum(timeforBUGSrun)
    BKtiming <- sum(timeforBKrun)
       

    firstorderMeanDiff = firstorder2 - firstorder1
    secondorderMeanDiff = secondorder2 - secondorder1
   
    list(MCMC_EVSI = TwolevelBUGSEVSI, SecondorderBK_EVSI = SecondorderBKEVSI, 
            BKversusMCMCRatio=BKversusTwoLevelRatio, BUGStiming=BUGStiming, BKtiming=BKtiming)
}



###
### code to send data to WinBUGS and get samples back
###

UnbalancedR2OpenBUGSfunction<-function(nposteriorsamples, nburnin, data)
{
#This code takes sample data, gives nposteriorsamples posterior samples, and then needs to undertake a PSA
sampledata<-PrepareDataForR2OpenBUGSfunction(data)
bugs.data(sampledata)  # this creates the file data.txt which BUGS uses

inits<-list(thetaM1=sampledata$theta0[c(6,15)],thetaM2=sampledata$theta0[c(5,7,14,16)])
inits<-list(inits)

parameters.to.save<-"theta"

UnbalancedBUGSresult<-bugs(data="data.txt", inits=inits, parameters.to.save,
                            model.file = "UnbalancedBUGSmodel.txt",
                            debug=FALSE, DIC=FALSE,digits=10,
				    version='312E',
				    working.directory=NULL,
                            n.chains = 1, n.iter = nposteriorsamples+nburnin, n.burnin = nburnin, n.thin=1)

UnbalancedBUGSresult.params<-UnbalancedBUGSresult$sims.list
testmean<-mean(UnbalancedBUGSresult.params$theta)
list(inits=inits, parameters.to.save=parameters.to.save,
UnbalancedBUGSresult=UnbalancedBUGSresult, UnbalancedBUGSresult.params=UnbalancedBUGSresult.params)
}



