### Jan 2011
###
### Code for functions needed for EVSI

# Load libraries
library(MASS)        # needed to sample from a multivariate normal (mvrnorm)
library(R2OpenBUGS)  # needed to call OpenBUGS from R and pass results back to R
# OpenBUGS and R2OpenBUGS are available from http://www.openbugs.info/w/

# We propose to have 'unblanced' data collection on the parameters
# This means that we will collect more data on some parameters than others
# We propose 3 separate datasets
#     Set 1 collects n1 samples on subset of parameters i1
#     Set 2 collects n2 samples on subset of parameters i2
#     Set 3 collects n3 samples on subset of parameters i3
# The three sets are iid.  But Set 1 and Set 2 might both collect data on the same parameters

# Define the parameters of interest
i1=c(5,14) # vector for the first i1 parameters # vector of subset of parameters Set 1 needed to collect n1 more data samples on
i2=c(6,15) # vector of subset of parameters Set 2 needed to collect n2 more data samples on
i3=c(7,16) # vector of subset of parameters Set 3 needed to collect n3 more data samples on
R<-c(5,6,7,14,15,16)

# Define few terms to speed things up
sigma0inverse <- solve(sigma0)
sigmainverse <- solve(sigma)
sigmai1=sigma[i1,i1]
sigmai1inverse <- solve(sigmai1)
sigmai2=sigma[i2,i2]
sigmai2inverse <- solve(sigmai2)
sigmai3=sigma[i3,i3]
sigmai3inverse <- solve(sigmai3)



##
## MAIN FUNCTION
##
#This function samples n1, n2, n3 patients given a sample of theta

data.sample <- function(theta, n1, n2, n3)
{
# This function is for calculating the data sample
# sigmai1, sigmai2, sigmai3, i1, i2, i3 are defined earlier
# n1 sample size for data on i1
# n2 sample size for data on i2 
# n3 for set i3

# Sample n1 data for thetai1
thetai1=theta[i1]
# cat("thetai1=", thetai1, "\n")
if(n1<1){datai1<-0
         meani1<-0}
if(n1>0)
    {datai1 <- mvrnorm(n1, mu = thetai1, Sigma = sigmai1)
    meani1 = apply(datai1, 2, mean)}

# Sample n2 data for thetai2
thetai2=theta[i2]
if(n2<1){datai2<-0
         meani2<-0}
if(n2>0)
    {datai2 <- mvrnorm(n2, mu = thetai2, Sigma = sigmai2)
     meani2 = apply(datai2, 2, mean)}

# Sample n3 data for thetai3
thetai3=theta[i3]
if(n3<1){datai3<-0
         meani3<-0}
if(n3>0)
    {datai3 <- mvrnorm(n3, mu = thetai3, Sigma = sigmai3)
     meani3 = apply(datai3, 2, mean)}

#Data preparation for OpenBUGS
# We want a matrix of NAs filled with the partial data
TotalN=n1+n2+n3
datacomplete <- array(NA, c(TotalN,d))
if(n1>0){for(i in 1:n1){  datacomplete[i,i1] <- datai1[i,]}}
if(n2>0){for(i in 1:n2){  datacomplete[i+n1,i2] <- datai2[i,]}}
if(n3>0){for(i in 1:n3){  datacomplete[i+n1+n2,i3] <- datai3[i,]}}

newdata=datacomplete

# data means
list(theta0=theta0, sigma0inverse=sigma0inverse, sigmainverse=sigmainverse,
        datai1 = datai1, datai2 = datai2, datai3 = datai3, meani1 = meani1, meani2 = meani2, meani3 = meani3, n1 = n1, n2 = n2, n3=n3, TotalN=TotalN, datai1 = datai1, datacomplete=datacomplete, newdata=newdata)
}



###
### Prepare data for OpenBUGS
###
# this gets the data.sample output into a form ready to go to R2OpenBUGS

PrepareDataForR2OpenBUGSfunction<-function(data)
{
TotalN <- data$TotalN
#theta0 <- data$theta0
#sigma0inverse <- data$sigma0inverse
#sigmainverse <- data$sigmainverse
dataM1 <- data$dataM1
dataM2 <- data$dataM2
sigmainvM1 <- data$sigmainvM1
sigmainvM2 <- data$sigmainvM2
sigma0invM1 <- data$sigma0invM1
sigma0invM2 <- data$sigma0invM2
theta0M1 <- data$theta0M1
theta0M2 <- data$theta0M2
list(TotalN=TotalN, dataM1=dataM1,dataM2=dataM2,
	sigmainvM1=sigmainvM1,sigmainvM2=sigmainvM2,sigma0invM1=sigma0invM1,sigma0invM2=sigma0invM2,theta0M1=theta0M1,theta0M2=theta0M2)
}



##########################################
# This does analytic Bayesian update for mulitvariate normal

#thetastarsigmastar
#produces the Bayesian update of the multivariate normal parameters - thetastar and sigmastar
# thetastar is the posterior mean and indeed the posterior mode since parameters are normal
# sigmastar is the posterior variance matrix
thetastarsigmastar <- function(theta0, sigma0, sigma, data)
{
# d is the paramter dimension
# nvector is a vector of n patients
# theta0 is the prior mean
# sigma0 is the prior var-covar matrix
# sigma is the data var-covar matrix

    d <- length(theta0)
    n <- length(data[,1])
    sigma0inverse <- solve(sigma0)
    sigmainverse <- solve(sigma)
    sigmastar <- solve(sigma0inverse + n * sigmainverse)
    xbar <- rep(0, len = d)
    for(i in 1:d) {
        xbar[i] <- mean(data[, i])
        }
    prod1 <- sigma0inverse %*% theta0
    prod2 <- n * sigmainverse %*% xbar
    thetastar <- sigmastar %*% (prod1 + prod2)
    list(thetastar = thetastar,sigmastar = sigmastar)
}



############################
#generalized.log.prior
# provides the prior function for a multivariate normal distribution

generalized.log.prior <- function(theta)
{
# theta is a vector that contains the parameters

     - t(theta - theta0) %*% sigma0inverse %*% (theta - theta0)/2
}

#generalized.log.lik
# provides the log likelihood fumction for a multivariate normal distribution
# (excludes constants

generalized.log.lik <- function(theta, data)
{
# theta is a vector that contains the parameters
# data is a list of three data means for three different data sets
# sigmai1, sigmai2, sigmai3, i1, i2, i3 are defined earlier

# meani1 is the vector of means for datai1
thetai1=theta[i1]
meani1 = data$meani1

# meani2 is the vector of means for datai2
thetai2=theta[i2]
meani2 = data$meani2

# meani3 is the vector of means for datai3
thetai3=theta[i3]
meani3 = data$meani3

# Sample sizes
n1 = data$n1
n2 = data$n2
n3 = data$n3

# The log likelihood fumction is
-n1 * t(thetai1 - meani1) %*% sigmai1inverse %*% (thetai1 - meani1)/2+
-n2 * t(thetai2 - meani2) %*% sigmai2inverse %*% (thetai2 - meani2)/2+
-n3 * t(thetai3 - meani3) %*% sigmai3inverse %*% (thetai3 - meani3)/2
}


generalized.log.posterior <- function(theta, data)
{
# theta is a vector that contains the parameters

generalized.log.prior(theta) + generalized.log.lik(theta, data)
}

#generalized.score.posterior
# provides the 1st derivative of the multivariate normal log likelohood function
# i.e. simply differentiates generalized.log.lik wrt to thetastar

generalized.score.posterior <- function(theta, data)
{
# theta is a vector that contains the parameters
# data is a list of three data means for three different data sets
# sigmai1, sigmai2, sigmai3, i1, i2, i3 are defined earlier

datai1score <- rep(0,len=d)
datai2score <- rep(0,len=d)
datai3score <- rep(0,len=d)

# meani1 is the vector of means for datai1
thetai1=theta[i1]
meani1 = data$meani1

# meani2 is the vector of means for datai2
thetai2=theta[i2]
meani2 = data$meani2

# meani3 is the vector of means for datai3
thetai3=theta[i3]
meani3 = data$meani3

# Sample sizes
n1 = data$n1
n2 = data$n2
n3 = data$n3

# prior score
priorscore <- - sigma0inverse %*% (theta - theta0)

# score for vector i1
i1score <- - n1 * sigmai1inverse %*% (thetai1 - meani1)
datai1score[i1] <- i1score

# score for vector i2
i2score <- - n2 * sigmai2inverse %*% (thetai2 - meani2)
datai2score[i2] <- i2score

# score for vector i3
i3score <- - n3 * sigmai3inverse %*% (thetai3 - meani3)
datai3score[i3] <- i3score

# The score function is then
priorscore + datai1score + datai2score + datai3score
}


