### Jan 2011
###
### Code for the decision model and the input distributions

# Input prior probability distributions and net benefit functions for the Decision Model parameters 

#  In this case study the uncertain parameters in the model "theta" 
#  have prior distribution that is multivariate normal with mean theta0 and variance matrix sigma0.
#
#  We assume that in a data collection exercise the individual level data will have 
#  mean theta and a 'patient level variance matrix' that is known and denoted sigma. 
#
#  There are 2 treatments with net benefit functions v1(theta) and v2(theta) respectively.

#mean
theta0=c(1000.00, 0.10, 5.20, 400.00, 0.70, 0.30, 3.00, 0.25, -0.10, 0.50, 1500.00, 0.08, 6.10, 0.80, 0.30, 3.00, 0.20, -0.10, 0.50)

d <- length(theta0)

#prior variance
sigma0=c(
 1.0e+000, 4.0e-004, 1.0e+000, 4.0e+004, 1.0e-002, 1.0e-002, 2.5e-001, 1.0e-002, 4.0e-004,
 4.0e-002, 1.0e+000, 4.0e-004, 1.0e+000, 1.0e-002, 2.5e-003, 1.0e+000, 2.5e-003, 4.0e-004,
 4.0e-002)

sigma0=diag(sigma0)

covariancefunction <-  function(correlation, var1, var2)
{correlation * sqrt(var1)* sqrt(var2)}

correlationtotest = 0.6

sigma0[5,14]=covariancefunction(correlationtotest, sigma0[5,5], sigma0[14,14])
sigma0[14,5]=sigma0[5,14]
sigma0[6,15]=covariancefunction(correlationtotest, sigma0[6,6], sigma0[15,15])
sigma0[15,6]=sigma0[6,15]
sigma0[5,7]=covariancefunction(correlationtotest, sigma0[5,5], sigma0[7,7])
sigma0[7,5]=sigma0[5,7]
sigma0[14,16]=covariancefunction(correlationtotest, sigma0[14,14], sigma0[16,16])
sigma0[16,14]=sigma0[14,16]
sigma0[7,14]=covariancefunction(correlationtotest, sigma0[7,7], sigma0[14,14])
sigma0[14,7]=sigma0[7,14]
sigma0[5,16]=covariancefunction(correlationtotest, sigma0[5,5], sigma0[16,16])
sigma0[16,5]=sigma0[5,16]
sigma0[7,16]=covariancefunction(correlationtotest, sigma0[7,7], sigma0[16,16])
sigma0[16,7]=sigma0[7,16]


# patient level variance for sample data in a data collection exercise
sigma=c(2.50e+005, 6.25e-002, 1.60e+001, 4.00e+004, 4.00e-002, 4.00e-002, 1.00e+000, 4.00e-002,
  1.00e-002, 6.40e-001, 2.50e+005, 6.25e-002, 1.60e+001, 4.00e-002, 4.00e-002, 4.00e+000,
  1.00e-002, 1.00e-002, 6.40e-001)

sigma=diag(sigma)

sigma[5,14]=covariancefunction(correlationtotest, sigma[5,5], sigma[14,14])
sigma[14,5]=sigma[5,14]
sigma[6,15]=covariancefunction(correlationtotest, sigma[6,6], sigma[15,15])
sigma[15,6]=sigma[6,15]
sigma[5,7]=covariancefunction(correlationtotest, sigma[5,5], sigma[7,7])
sigma[7,5]=sigma[5,7]
sigma[14,16]=covariancefunction(correlationtotest, sigma[14,14], sigma[16,16])
sigma[16,14]=sigma[14,16]
sigma[7,14]=covariancefunction(correlationtotest, sigma[7,7], sigma[14,14])
sigma[14,7]=sigma[7,14]
sigma[5,16]=covariancefunction(correlationtotest, sigma[5,5], sigma[16,16])
sigma[16,5]=sigma[5,16]
sigma[7,16]=covariancefunction(correlationtotest, sigma[7,7], sigma[16,16])
sigma[16,7]=sigma[7,16]


# Determinant of a matrix
det <- function(x)
prod(eigen(x)$values)

lambda<-10000

# Define function v1 = Net Benefit1
v1 <- function(theta, lambda)
{# theta is vector that contains the d parameters,  lambda is threshold cost per QALY
    lambda * (theta[5] * theta[6] * theta[7] + theta[8] * theta[9] * theta[10]) - (theta[1] + theta[2] * theta[3] * theta[4])
}

# Define function v2 = Net Benefit2
v2 <- function(theta, lambda)
{
    lambda * (theta[14] * theta[15] * theta[16] + theta[17] * theta[18] * theta[19]) - (theta[11] + theta[12] * theta[13] * theta[4])
}


# Define function bldecision that computes the baseline decision and overall evpi
bldecision <- function(theta0, sigma0, lambda, ndraw)
{   NB1 <- rep(0, len = ndraw)
    NB2 <- rep(0, len = ndraw)
    maxNB <- rep(0, len = ndraw)
    for(i in 1:ndraw) {
                        print(i)
                        flush.console()

                        theta <- mvrnorm(1, mu = theta0, Sigma = sigma0)
                        NB1[i] <- v1(theta, lambda)
                        NB2[i] <- v2(theta, lambda)
                        maxNB[i] <- max(NB1[i],NB2[i])
                        }
    mean1 <- mean(NB1)
    mean2 <- mean(NB2)
    baseline <- max(mean1, mean2)
    overallevpi<- mean(maxNB)-max(mean1, mean2)
    list(baseline=baseline, overallevpi=overallevpi)
}

# Running the above function we get the maximum expected net benefit of the 2 treatments 
# i.e. the value of the baseline adoption decision

#PSAresult <- bldecision(theta0, sigma0, lambda, 1000000)
#baseline <- PSAresult$baseline
#overallevpi <- PSAresult$overallevpi


# > PSAresult $baseline [1] 5590.895     $overallevpi  [1] 844.4438

# based on 1000000 samples we get
baseline <- 5590.895 
overallevpi <- 844.4438
